require "james_bond/core/mission/undefined_variable_error"

module JamesBond
  module Core
    module Mission
      MissionConfig = Struct.new("MissionConfig", :name, :main_commands)

      attr_accessor :core_mission_options,
                    :core_mission_config

      attr_reader   :option_rules

      def initialize(params={})
        @core_mission_config  = MissionConfig.new
        @core_mission_options = {}

        build(@core_mission_config, params)
        check_variables_definitions
        check_main_command_methods
      end

      def build(config, params)
        raise(NotImplementedError, "The method \"build\" must be implemented for the #{self.class} class")
      end

      def run_command(command: , mission_pool:)
        @core_mission_options = command.options = parse_arguments(command) if @option_rules
        command.options = parse_arguments(command) if @option_rules
        self.send(
          build_command_method_name(command.main_command),
          { command: command, mission_pool: mission_pool }
        )
      end

      def arguments_parser(&block)
        require "slop"
        @option_rules = Slop::Options.new
        yield(@option_rules)
      end

      private

      def parse_arguments(command)
        arguments_array = command.raw_arguments
        Slop::Parser.new(@option_rules).parse(arguments_array)
      end

      def build_command_method_name(main_command)
        "run_#{main_command}_command"
      end

      def check_variables_definitions
        raise(UndefinedVariableError,
              "The mission name must be defined in the build method. " \
            + "Try calling config.name = \"MissionName\" inside the build method") if !@core_mission_config.name
        raise(UndefinedVariableError,
              "The main commands must be defined in the build method. " \
            + "Try calling config.main_commands = [\"cmd\", \"dmc\"] inside the build method") if !@core_mission_config.main_commands
      end

      def check_main_command_methods
        @core_mission_config.main_commands.each do |main_command|
          expected_command_method = build_command_method_name(main_command)
          raise(NotImplementedError,
                "The method \"#{expected_command_method}\" must be implemented for the #{self.class}") if !self.respond_to?(expected_command_method)
        end
      end
    end
  end
end
