module JamesBond
  module Core
    class MissionPool
      attr_accessor :mission_list, :mission_by_main_command

      def initialize(params={})
        @mission_list = params[:mission_list]
        build_mission_by_main_command
      end

      def decide_mission(command)
        @mission_by_main_command[command.main_command]
      end

      private

      def build_mission_by_main_command
        @mission_by_main_command = {}
        @mission_list.each do |mission|
          mission.core_mission_config.main_commands.each do |main_command|
            another_mission = @mission_by_main_command[main_command]
            raise "Mission conflict! Two missions (\"#{another_mission.name}\" and \"#{mission.name}\") " \
                + "are assigned for the same command (\"#{main_command}\")" if another_mission
            @mission_by_main_command[main_command] = mission
          end
        end
      end
    end
  end
end
